package eu.ztsh.wymiana;

import eu.ztsh.wymiana.data.entity.CurrencyEntity;
import eu.ztsh.wymiana.data.entity.UserEntity;
import eu.ztsh.wymiana.web.model.UserCreateRequest;

import java.util.ArrayList;
import java.util.List;
import java.util.Optional;

public class EntityCreator {

    public static class Constants {

        public static String PESEL = "00281018264";
        public static String NAME = "Janina";
        public static String SURNAME = "Kowalska";
        public static double PLN = 20.10;

    }

    public static UserEntityBuilder user() {
        return new UserEntityBuilder();
    }

    public static UserCreateRequest.UserCreateRequestBuilder userRequest() {
        return UserCreateRequest.builder().name(Constants.NAME)
            .surname(Constants.SURNAME)
            .pesel(Constants.PESEL)
            .pln(Constants.PLN);
    }

    public static class UserEntityBuilder {

        String name;
        String surname;
        String pesel;
        double pln;
        double usd;

        public UserEntityBuilder name(String name) {
            this.name = name;
            return this;
        }

        public UserEntityBuilder surname(String surname) {
            this.surname = surname;
            return this;
        }

        public UserEntityBuilder pesel(String pesel) {
            this.pesel = pesel;
            return this;
        }

        public UserEntityBuilder pln(double pln) {
            this.pln = pln;
            return this;
        }

        public UserEntityBuilder usd(double usd) {
            this.usd = usd;
            return this;
        }

        public UserEntity build() {
            var nonnulPesel = Optional.ofNullable(pesel).orElse(Constants.PESEL);
            List<CurrencyEntity> currencies = new ArrayList<>();
            if (pln > 0) {
                currencies.add(new CurrencyEntity(nonnulPesel, "PLN", pln));
            }
            if (usd > 0) {
                currencies.add(new CurrencyEntity(nonnulPesel, "USD", usd));
            }
            if (currencies.isEmpty()) {
                currencies.add(new CurrencyEntity(nonnulPesel, "PLN", Constants.PLN));
            }
            return new UserEntity(
                nonnulPesel,
                Optional.ofNullable(name).orElse(Constants.NAME),
                Optional.ofNullable(surname).orElse(Constants.SURNAME),
                currencies
            );
        }

    }

}
